//
// Device functions for InnoSetup 5.x
//

const
  DELETE_ACCESS_RIGHT = $00010000;
  READ_CONTROL        = $00020000;
  WRITE_DAC           = $00040000;
  WRITE_OWNER         = $00080000;
  SYNCHRONIZE         = $00100000;
  
function SimpleCreateDevice(ADeviceName, ADisplayName, AFileName: string;
  AStartType: Longword; ADesiredAccess: Longword; AnErrorControl: Longword;
  IgnoreExisting: Boolean): Boolean;
var
  SCMHandle: Longword;
  ServiceHandle: Longword;
  ServiceType: Longword;
  dwTag: Longword;
  Error: Integer;
begin
  Result := False;
  dwTag := 0;
  ServiceType := SERVICE_KERNEL_DRIVER;
  try
    SCMHandle := OpenSCManager('', '', SC_MANAGER_ALL_ACCESS);
    if SCMHandle = 0 then
      RaiseException('OpenSCManager: ' + ADeviceName + ' ' + SysErrorMessage(DLLGetLastError));
    try
      ServiceHandle := CreateService(SCMHandle, ADeviceName, ADisplayName, ADesiredAccess, ServiceType,
        AStartType, AnErrorControl, AFileName, '', 0, '', '', '');
      if ServiceHandle = 0 then
      begin
        Error := DLLGetLastError;
        if IgnoreExisting and (Error = ERROR_SERVICE_EXISTS) then
          Exit
        else
          RaiseException('CreateService: ' + ADeviceName + ' ' + SysErrorMessage(Error));
      end;
      Result := True;
    finally
      if ServiceHandle <> 0 then
        CloseServiceHandle(ServiceHandle);
    end;
  finally
    if SCMHandle <> 0 then
      CloseServiceHandle(SCMHandle);
  end;
end;

procedure SimpleStopDevice(ADevice: string; ADesiredAccess: Longword; Wait, IgnoreStopped: Boolean);
var
  DeviceStatus: _SERVICE_STATUS;
  SCMHandle: Longword;
  DeviceHandle: Longword;
  Error: Integer;
begin
  try
    SCMHandle := OpenSCManager('', '', SC_MANAGER_ALL_ACCESS);
    if SCMHandle = 0 then
      RaiseException('OpenSCManager: ' + ADevice + ' ' + SysErrorMessage(DLLGetLastError));
    try
      DeviceHandle := OpenService(SCMHandle, ADevice, ADesiredAccess);
      if DeviceHandle = 0 then
        RaiseException('OpenService: ' + ADevice + ' ' + SysErrorMessage(DLLGetLastError));
      try
        if ControlService(DeviceHandle, SERVICE_CONTROL_STOP, DeviceStatus) = 0 then
        begin
          Error := DLLGetLastError;
          if IgnoreStopped and (Error = ERROR_SERVICE_NOT_ACTIVE) then
            Exit
          else
            RaiseException('ControlService: ' + ADevice + ' ' + SysErrorMessage(Error));
          if Wait then
            WaitForService(DeviceHandle, SERVICE_STOPPED);
        end;
      finally
        if DeviceHandle <> 0 then
          CloseServiceHandle(DeviceHandle);
      end;
    finally
      if SCMHandle <> 0 then
        CloseServiceHandle(SCMHandle);
    end;
  except
    ShowExceptionMessage;
  end;
end;

procedure SimpleStartDevice(ADevice: string; ADesiredAccess: Longword;
  Wait, IgnoreStarted: Boolean);
var
  SCMHandle: Longword;
  DeviceHandle: Longword;
  Error: Integer;
begin
  try
    SCMHandle := OpenSCManager('', '', SC_MANAGER_ALL_ACCESS);
    if SCMHandle = 0 then
      RaiseException('OpenSCManager: ' + ADevice + ' ' + SysErrorMessage(DLLGetLastError));
    try
      DeviceHandle := OpenService(SCMHandle, ADevice, ADesiredAccess);
      if DeviceHandle = 0 then
        RaiseException('OpenService: ' + ADevice + ' ' + SysErrorMessage(DLLGetLastError));
      try
        if StartService(DeviceHandle, 0, '') = 0 then
        begin
          Error := DLLGetLastError;
          if IgnoreStarted and (Error = ERROR_SERVICE_ALREADY_RUNNING) then
            Exit
          else
            RaiseException('StartDevice: ' + ADevice + ' ' + SysErrorMessage(Error));
          if Wait then
          begin
            WaitForService(DeviceHandle, SERVICE_RUNNING);
          end;
        end;
      finally
        if DeviceHandle <> 0 then
          CloseServiceHandle(DeviceHandle);
      end;
    finally
      if SCMHandle <> 0 then
        CloseServiceHandle(SCMHandle);
    end;
  except
    ShowExceptionMessage;
  end;
end;

procedure SimpleDeleteDevice(ADevice: string; ADesiredAccess: Longword);
var
  SCMHandle: Longword;
  DeviceHandle: Longword;
begin
  try
    SCMHandle := OpenSCManager('', '', SC_MANAGER_ALL_ACCESS);
    if SCMHandle = 0 then
      RaiseException('OpenSCManager: ' + ADevice + ' ' + SysErrorMessage(DLLGetLastError));
    try
      DeviceHandle := OpenService(SCMHandle, ADevice, ADesiredAccess);
      if DeviceHandle = 0 then
        RaiseException('OpenService: ' + ADevice + ' ' + SysErrorMessage(DLLGetLastError));
      try
        if DeleteService(DeviceHandle) = 0 then
          RaiseException('DeleteService: ' + ADevice + ' ' + SysErrorMessage(DLLGetLastError));
      finally
        if DeviceHandle <> 0 then
          CloseServiceHandle(DeviceHandle);
      end;
    finally
      if SCMHandle <> 0 then
        CloseServiceHandle(SCMHandle);
    end;
  except
    ShowExceptionMessage;
  end;
end;

function DeviceExists(ADevice: string; ADesiredAccess: Longword): Boolean;
var
  SCMHandle: Longword;
  DeviceHandle: Longword;
  Error: Integer;
begin
  try
    SCMHandle := OpenSCManager('', '', SC_MANAGER_ALL_ACCESS);
    if SCMHandle = 0 then
      RaiseException('OpenSCManager: ' + ADevice + ' ' + SysErrorMessage(DLLGetLastError));
    try
      DeviceHandle := OpenService(SCMHandle, ADevice, ADesiredAccess);
      try
        if DeviceHandle = 0 then
        begin
          Error := DLLGetLastError;
          if Error = ERROR_SERVICE_DOES_NOT_EXIST then
            Result := False
          else
            RaiseException('OpenService: ' + ADevice + ' ' + SysErrorMessage(Error));
        end
        else
          Result := True;
      finally
        if DeviceHandle <> 0 then
          CloseServiceHandle(DeviceHandle);
      end;
    finally
      if SCMHandle <> 0 then
        CloseServiceHandle(SCMHandle);
    end;
  except
    ShowExceptionMessage;
  end;
end;

function SimpleQueryDevice(ADevice: string; ADesiredAccess: Longword): Longword;
var
  DeviceStatus: _SERVICE_STATUS;
  SCMHandle: Longword;
  DeviceHandle: Longword;
  Error: Integer;
begin
  Result := 0;
  try
    SCMHandle := OpenSCManager('', '', SC_MANAGER_ALL_ACCESS);
    if SCMHandle = 0 then
      RaiseException('OpenSCManager: ' + ADevice + ' ' + SysErrorMessage(DLLGetLastError));
    try
      DeviceHandle := OpenService(SCMHandle, ADevice, ADesiredAccess);
      if DeviceHandle = 0 then
        RaiseException('OpenService: ' + ADevice + ' ' + SysErrorMessage(DLLGetLastError));
      try
        if ControlService(DeviceHandle, SERVICE_CONTROL_INTERROGATE, DeviceStatus) = 0 then
        begin
          Error := DLLGetLastError;
          RaiseException('ControlService: ' + ADevice + ' ' + SysErrorMessage(Error));
        end;
        Result := DeviceStatus.dwCurrentState;
      finally
        if DeviceHandle <> 0 then
          CloseServiceHandle(DeviceHandle);
      end;
    finally
      if SCMHandle <> 0 then
        CloseServiceHandle(SCMHandle);
    end;
  except
    ShowExceptionMessage;
  end;
end;
